package suicideChess;

import suicideChess.Square.NotAValidSquare;

/**
 * This class is used for moves representation.
 * 
 * @author Jean-Baptiste H&eacute;tier
 * @version $LastChangedRevision$, $LastChangedDate$
 */

public class Move {

	/*======*
	 * DATA *
	 *======*/
	
	//integers 
	private Piece movingPiece;

	private Square fromSquare;
	private Square toSquare;
	private Piece promotionPiece;
	private boolean isPromotion;
	
	private boolean isCapture;
	private Piece capturePiece;

	public class NotAValidMoveException extends Exception {
		/**
		 * Generated by Eclipse
		 */
		private static final long serialVersionUID = 2194133427162274651L;

		NotAValidMoveException(String s) { super(s); };
	}
	
	/*=============*
	 * CONSTRUCTOR *
	 *=============*/

	/**
	 * This is the constructor of the class.
	 * 
	 * @param move A String containing of type "e2e4" (4 chars), "b7b8q" (5 chars) for promotions.
	 * It is the standard algebraic notation used for chess.
	 * @param board A {@link Board} to be able to locate the pieces.
	 * @throws NotAValidMoveException When the String is not legal (too long or too short).
	 * @throws NotAValidSquare It the String is not legal (not standard algebraic notation).
	 * @see Board
	 */
	public Move(String move, Board board) throws NotAValidMoveException, NotAValidSquare {
		isPromotion = false;
		
		switch (move.length()) {
			case 5:
				isPromotion = true;
				//no break statement here on purpose
			case 4:
				fromSquare = new Square(move.substring(0,2));
				toSquare = new Square(move.substring(2,4));
				break;
			default:
				throw new NotAValidMoveException("Invalid Move: "+move);
		}
		
		movingPiece = board.getPiece(fromSquare());
		capturePiece = board.getPiece(toSquare());
		
		if (capturePiece.getPieceNumber()!=Piece.NONE) {
			isCapture = true;
		} else {
			isCapture = false;
		}
		
		if (isPromotion) {
			promotionPiece = new Piece(move.toCharArray()[4], movingPiece.getColor());
		}
	}
	
	
	/*===========================================*
	 * Those two constructors are no longer used *
	 *===========================================*/
	/*
	//non capture move
	public Move(String move, Piece pieceToMove) throws NotAValidMoveException, NotAValidSquare {
		movingPiece = pieceToMove;
		isPromotion = false;
		isCapture = false;
		
		switch (move.length()) {
			case 5:
				isPromotion = true;
				promotionPiece = new Piece(move.toCharArray()[4], movingPiece.getColor());
				//no break statement here on purpose
			case 4:
				fromSquare = new Square(move.substring(0,2));
				toSquare = new Square(move.substring(2,4));
				break;
			default:
				throw new NotAValidMoveException("Invalid Move: "+move);
		}
	}
	
	//capture move
	public Move(String move, Piece pieceToMove, Piece pieceToCapture) throws NotAValidMoveException, NotAValidSquare {
		movingPiece = pieceToMove;
		isPromotion = false;
		isCapture = true;
		capturePiece = pieceToCapture;
		
		switch (move.length()) {
			case 5:
				isPromotion = true;
				promotionPiece = new Piece(move.toCharArray()[4], movingPiece.getColor());
				//no break statement here on purpose
			case 4:
				fromSquare = new Square(move.substring(0,2));
				toSquare = new Square(move.substring(2,4));
				break;
			default:
				throw new NotAValidMoveException("Invalid Move: "+move);
		}
	}

	*/
	
	
	/*=========*
	 * METHODS *
	 *=========*/
	
	/**
	 * Returns the {@link Square} to move from in a Move
	 * @return Square
	 * @see Square 
	 */
	public Square fromSquare() {
		return fromSquare;
	}

	/**
	 * Returns the {@link Square} to move to in a Move
	 * @return Square
	 * @see Square 
	 */
	public Square toSquare() {
		return toSquare;
	}
	
	/**
	 * Returns a boolean saying if a Move is a promotion or not.
	 * @return boolean
	 */
	public boolean isPromotionMove() {
		return isPromotion;
	}
	
	/**
	 * Returns the promotion {@link Piece} of a Move
	 * @return Piece
	 * @see Piece
	 */
	public Piece getPromotionPiece() {
		if (isPromotion) {
			return promotionPiece;
		} else {
			return new Piece(Piece.NONE);
		}
	}
	
	/**
	 * Returns the moving {@link piece} of a Move.
	 * @return Piece
	 * @see Piece
	 */
	public Piece getMovingPiece() {
		return movingPiece;
	}
	
	/** 
	 * Returns a boolean saying if a Move is a capture move
	 * @return boolean
	 */
	public boolean isCaptureMove() {
		return isCapture;
	}
	
	/**
	 * Returns the {@link Piece} to be captured in a Move 
	 * @return Piece
	 */
	public Piece getCapturedPiece() {
		return capturePiece;
	}
	
	/**
	 * Converts a Move into a String.
	 * @return String 
	 */
	public String toString() {
		return fromSquare.toString()+toSquare+promotionPiece;
	}

	/**
	 * Displays a move in great details.
	 */
	public void display() {
		System.out.println(" ");
		System.out.println("==== Move ====");
		System.out.println("Move:       "+getMovingPiece().getPieceNumber());
		System.out.println("From square:"+fromSquare());
		System.out.println("To square:  "+toSquare());
		System.out.println((isPromotionMove() ? "Promotion:  "+getPromotionPiece().getPieceNumber() : "No Promotion"));
		System.out.println((isCaptureMove() ? "Capture:    "+getCapturedPiece().getPieceNumber() : "No Capture"));
		System.out.println("==============");
		System.out.println(" ");
		System.out.println(" ");
	}
	
	
	/*======================================*
	 * I do not use those functions anymore *
	 *======================================*
	
	public void setFromSquare(Square square) {
		fromSquare = square;
	}
	public void setToSquare(Square square) {
		toSquare = square;
	}
	public void setPromotionMove(String piece) {
		if (piece.length()==0) {
			isPromotion=false;
		} else {
			isPromotion=true;
			promotionPiece=new Piece(piece);
		}
	}
	public void setMovingPiece(Piece piece) {
		movingPiece = piece;
	}
		
	*/
}
